package com.alibaba.cloud.security.sample.service;

import com.alibaba.cloud.security.sample.entity.UserDO;
import com.google.gson.Gson;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * 由于本Demo项目关注的是认认证，所以用户模块只开发了Mock实现
 */
@Service
public class MockUserService {
    private ConcurrentMap<String, UserDO> userMap = new ConcurrentHashMap<>();
    private ConcurrentMap<String, String> userIdMap = new ConcurrentHashMap<>();

    public MockUserService() {
        try {
            loadFromFile();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    synchronized public String createUser(UserDO user) {
        String loginId = user.getLoginId();

        if (!StringUtils.hasText(loginId)) {
            throw new IllegalStateException("User login id cannot be empty");
        }
        if (userIdMap.containsKey(loginId)) {
            throw new IllegalStateException("User login id exists");
        }

        String userId = UUID.randomUUID().toString();
        user.setUserId(userId);
        userMap.put(userId, user);
        userIdMap.put(loginId, userId);

        writeToFile(user);

        return userId;
    }

    public UserDO findUserByUserId(String guid) {
        return userMap.get(guid);
    }

    public UserDO findUserByLoginId(String loginId) {
        String userId = userIdMap.get(loginId);
        return (userId == null) ? null : findUserByUserId(userId);
    }

    // --------------------------------------------
    static final File baseDir = new File("__user_cache_dir");

    synchronized void writeToFile(UserDO user) {
        if (user != null) {
            baseDir.mkdirs();
            File f = new File(baseDir, user.getUserId());
            try (OutputStream os = new FileOutputStream(f)) {
                os.write(new Gson().toJson(user).getBytes(StandardCharsets.UTF_8));
            } catch (Exception e) {
                // DEMO应用，所以忽略异常
                e.printStackTrace();
            }
        }
    }

    synchronized void loadFromFile() {
        if (baseDir.exists()) {
            File[] files = baseDir.listFiles();
            if (files != null) {
                for (File f : files) {
                    try (InputStream is = new FileInputStream(f)) {
                        try (InputStreamReader isr = new InputStreamReader(is, StandardCharsets.UTF_8)) {
                            UserDO user = new Gson().fromJson(isr, UserDO.class);
                            userMap.put(user.getUserId(), user);
                            userIdMap.put(user.getLoginId(), user.getUserId());
                        }
                    } catch (Exception e) {
                        // DEMO应用，所以忽略异常
                        e.printStackTrace();
                    }
                }
            }
        }
    }
}
