package com.alibaba.cloud.security.sample.web;

import com.alibaba.cloud.security.sample.entity.UserDO;
import com.alibaba.cloud.security.sample.params.authentication.*;
import com.alibaba.cloud.security.sample.service.IDaaSClientService;
import com.alibaba.cloud.security.sample.service.MockUserService;
import com.alibaba.cloud.security.sample.util.LoginUtil;
import com.aliyuncs.idaas_doraemon.model.v20210520.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.List;

@Api
@Controller
@RequestMapping("/authentication")
public class AuthenticationController {
    @Value("${app.idaas.applicationExternalId}")
    String alibabaCloudIdaasApplicationExternalId;

    @Autowired
    MockUserService mockUserService;
    @Autowired
    IDaaSClientService iDaaSClientService;

    @ApiOperation(value = "ListAuthenticators", notes = "List user's registered authenticators")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public ResponseEntity<ListAuthenticatorResponse> list(@ApiIgnore HttpSession session, @RequestBody ListAuthenticatorRequest listAuthenticatorRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);

        ListAuthenticatorsRequest listAuthenticatorsRequest = new ListAuthenticatorsRequest();
        listAuthenticatorsRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        listAuthenticatorsRequest.setUserId(userId);
        listAuthenticatorsRequest.setPageSize(100L);

        ListAuthenticatorsResponse listAuthenticatorsResponse = iDaaSClientService.getClient().getAcsResponse(listAuthenticatorsRequest);

        List<Authenticator> returnAuthenticators = new ArrayList<>();
        List<ListAuthenticatorsResponse.AuthenticatorListDTO> authenticators = listAuthenticatorsResponse.getAuthenticator();
        if (authenticators != null && authenticators.size() > 0) {
            for (ListAuthenticatorsResponse.AuthenticatorListDTO authenticatorListDTO : authenticators) {
                Authenticator authenticator = new Authenticator();
                authenticator.setType(authenticatorListDTO.getType());
                authenticator.setUuid(authenticatorListDTO.getAuthenticatorUuid());
                authenticator.setName(authenticatorListDTO.getAuthenticatorName());
                authenticator.setRegisterTime(authenticatorListDTO.getRegisterTime());
                authenticator.setLastVerifyTime(authenticatorListDTO.getLastVerifyTime());

                returnAuthenticators.add(authenticator);
            }
        }

        ListAuthenticatorResponse listAuthenticatorResponse = new ListAuthenticatorResponse();
        listAuthenticatorResponse.setAuthenticators(returnAuthenticators);

        return ResponseEntity.ok(listAuthenticatorResponse);
    }

    @ApiOperation(value = "DeleteAuthenticator", notes = "Delete one user's registered authenticator")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public ResponseEntity<DeleteAuthenticatorResponse> deleteAuthenticator(@ApiIgnore HttpSession session, @RequestBody DeleteAuthenticatorRequest deleteAuthenticatorRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);

        DeregisterAuthenticatorRequest deregisterAuthenticatorRequest = new DeregisterAuthenticatorRequest();
        deregisterAuthenticatorRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        deregisterAuthenticatorRequest.setUserId(userId);
        deregisterAuthenticatorRequest.setAuthenticatorUuid(deleteAuthenticatorRequest.getUuid());

        DeregisterAuthenticatorResponse deregisterAuthenticatorResponse = iDaaSClientService.getClient().getAcsResponse(deregisterAuthenticatorRequest);

        DeleteAuthenticatorResponse deleteAuthenticatorResponse = new DeleteAuthenticatorResponse();

        return ResponseEntity.ok(deleteAuthenticatorResponse);
    }

    @ApiOperation(value = "RenameAuthenticator", notes = "Rename user's registered authenticator")
    @RequestMapping(value = "/rename", method = RequestMethod.POST)
    public ResponseEntity<RenameAuthenticatorResponse> renameAuthenticator(@ApiIgnore HttpSession session, @RequestBody RenameAuthenticatorRequest renameAuthenticatorRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);

        UpdateAuthenticatorAttributeRequest updateAuthenticatorAttributeRequest = new UpdateAuthenticatorAttributeRequest();
        updateAuthenticatorAttributeRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        updateAuthenticatorAttributeRequest.setUserId(userId);
        updateAuthenticatorAttributeRequest.setAuthenticatorUuid(renameAuthenticatorRequest.getUuid());
        updateAuthenticatorAttributeRequest.setAuthenticatorName(renameAuthenticatorRequest.getName());

        UpdateAuthenticatorAttributeResponse updateAuthenticatorAttributeResponse = iDaaSClientService.getClient().getAcsResponse(updateAuthenticatorAttributeRequest);

        RenameAuthenticatorResponse renameAuthenticatorResponse = new RenameAuthenticatorResponse();

        return ResponseEntity.ok(renameAuthenticatorResponse);
    }

    @RequestMapping(value = "/initialize_create_credential", method = RequestMethod.POST)
    public ResponseEntity<InitializeCredentialResponse> initializeCreateCredential(@ApiIgnore HttpSession session, @RequestBody InitializeCreateCredentialRequest initializeCreateCredentialRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);
        UserDO user = null;
        if (StringUtils.hasText(userId)) {
            user = mockUserService.findUserByUserId(userId);
        }
        if (user == null) {
            return ResponseEntity.notFound().build();
        }

        CreateAuthenticatorRegistrationRequest createAuthenticatorRegistrationRequest = new CreateAuthenticatorRegistrationRequest();
        createAuthenticatorRegistrationRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        createAuthenticatorRegistrationRequest.setAuthenticatorType("WEBAUTHN");
        createAuthenticatorRegistrationRequest.setUserId(userId);
        createAuthenticatorRegistrationRequest.setUserName(user.getLoginId());
        createAuthenticatorRegistrationRequest.setUserDisplayName(user.getDisplayName());

        CreateAuthenticatorRegistrationResponse createAuthenticatorRegistrationResponse = iDaaSClientService.getClient().getAcsResponse(createAuthenticatorRegistrationRequest);

        InitializeCredentialResponse initializeCredentialResponse = new InitializeCredentialResponse();
        initializeCredentialResponse.setOptions(createAuthenticatorRegistrationResponse.getOptions());
        return ResponseEntity.ok(initializeCredentialResponse);
    }

    @RequestMapping(value = "/create_credential", method = RequestMethod.POST)
    public ResponseEntity<CreateCredentialResponse> createCredential(@ApiIgnore HttpSession session, @RequestBody CreateCredentialRequest createCredentialRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);
        UserDO user = null;
        if (StringUtils.hasText(userId)) {
            user = mockUserService.findUserByUserId(userId);
        }
        if (user == null) {
            return ResponseEntity.notFound().build();
        }

        RegisterAuthenticatorRequest registerAuthenticatorRequest = new RegisterAuthenticatorRequest();
        registerAuthenticatorRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        registerAuthenticatorRequest.setAuthenticatorType("WEBAUTHN");
        registerAuthenticatorRequest.setUserId(userId);
        registerAuthenticatorRequest.setAuthenticatorName(createCredentialRequest.getName());
        registerAuthenticatorRequest.setRegistrationContext(createCredentialRequest.getRegistrationContext());

        RegisterAuthenticatorResponse registerAuthenticatorResponse = iDaaSClientService.getClient().getAcsResponse(registerAuthenticatorRequest);

        String authenticatorUuid = registerAuthenticatorResponse.getAuthenticatorUuid();

        CreateCredentialResponse createCredentialResponse = new CreateCredentialResponse();
        createCredentialResponse.setAuthenticatorUuid(authenticatorUuid);
        return ResponseEntity.ok(createCredentialResponse);
    }

    @RequestMapping(value = "/initialize_verify_credential", method = RequestMethod.POST)
    public ResponseEntity<InitializeVerifyCredentialResponse> initializeVerifyCredential(@RequestBody InitializeVerifyCredentialRequest initializeVerifyCredentialRequest) throws Exception {
        CreateUserAuthenticateOptionsRequest createUserAuthenticateOptionsRequest = new CreateUserAuthenticateOptionsRequest();
        createUserAuthenticateOptionsRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        createUserAuthenticateOptionsRequest.setAuthenticatorType("WEBAUTHN");
        createUserAuthenticateOptionsRequest.setUserId(initializeVerifyCredentialRequest.getUserId());

        CreateUserAuthenticateOptionsResponse createUserAuthenticateOptionsResponse = iDaaSClientService.getClient().getAcsResponse(createUserAuthenticateOptionsRequest);

        InitializeVerifyCredentialResponse initializeVerifyCredentialResponse = new InitializeVerifyCredentialResponse();
        initializeVerifyCredentialResponse.setOptions(createUserAuthenticateOptionsResponse.getOptions());
        return ResponseEntity.ok(initializeVerifyCredentialResponse);
    }

    @RequestMapping(value = "/verify_credential", method = RequestMethod.POST)
    public ResponseEntity<VerifyCredentialResponse> verifyCredential(@ApiIgnore HttpSession session, @RequestBody VerifyCredentialRequest verifyCredentialRequest) throws Exception {
        VerifyUserAuthenticationRequest verifyUserAuthenticationRequest = new VerifyUserAuthenticationRequest();
        verifyUserAuthenticationRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        verifyUserAuthenticationRequest.setAuthenticatorType("WEBAUTHN");
        verifyUserAuthenticationRequest.setUserId(verifyCredentialRequest.getUserId());
        verifyUserAuthenticationRequest.setAuthenticationContext(verifyCredentialRequest.getRegistrationContext());

        VerifyUserAuthenticationResponse verifyUserAuthenticationResponse = iDaaSClientService.getClient().getAcsResponse(verifyUserAuthenticationRequest);

        boolean verifyResult = (verifyUserAuthenticationResponse.getVerifyResult() != null) && verifyUserAuthenticationResponse.getVerifyResult();

        VerifyCredentialResponse verifyCredentialResponse = new VerifyCredentialResponse();
        if (verifyResult) {
            verifyCredentialResponse.setLoginSuccess(true);
            LoginUtil.login(session, verifyCredentialRequest.getUserId());
        } else {
            verifyCredentialResponse.setLoginSuccess(false);
        }
        return ResponseEntity.ok(verifyCredentialResponse);
    }

    @ApiOperation(value = "ListAuthentication Logs", notes = "List user's authentication logs")
    @RequestMapping(value = "/list_log", method = RequestMethod.POST)
    public ResponseEntity<ListAuthenticationLogResponse> listLogs(@ApiIgnore HttpSession session, @RequestBody ListAuthenticationLogRequest listAuthenticationLogRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);

        ListAuthenticationLogsRequest listAuthenticationLogsRequest = new ListAuthenticationLogsRequest();
        listAuthenticationLogsRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        listAuthenticationLogsRequest.setUserId(userId);
        listAuthenticationLogsRequest.setPageSize(10L);

        List<AuthenticationLog> authenticationLogs = new ArrayList<>();
        ListAuthenticationLogsResponse listAuthenticationLogsResponse = iDaaSClientService.getClient().getAcsResponse(listAuthenticationLogsRequest);
        if (listAuthenticationLogsResponse.getAuthenticationLogContent() != null) {
            for (ListAuthenticationLogsResponse.AuthenticationLogContentItem item : listAuthenticationLogsResponse.getAuthenticationLogContent()) {
                AuthenticationLog authenticationLog = new AuthenticationLog();
                authenticationLog.setName(item.getAuthenticatorName());
                authenticationLog.setAuthenticatorType(item.getAuthenticatorType());
                authenticationLog.setAuthenticationTime(item.getAuthenticationTime());
                authenticationLog.setSuccess(item.getVerifyResult());
                authenticationLog.setUserAgent(item.getUserAgent());
                authenticationLog.setUserSourceIp(item.getUserSourceIp());
                authenticationLogs.add(authenticationLog);
            }
        }

        ListAuthenticationLogResponse listAuthenticationLogResponse = new ListAuthenticationLogResponse();
        listAuthenticationLogResponse.setAuthenticationLogs(authenticationLogs);

        return ResponseEntity.ok(listAuthenticationLogResponse);
    }

    @ApiOperation(value = "ListAuthenticator Ops Logs", notes = "List user's authenticators ops logs")
    @RequestMapping(value = "/list_ops_log", method = RequestMethod.POST)
    public ResponseEntity<ListAuthenticatorOpsLogResponse> listOpsLogs(@ApiIgnore HttpSession session, @RequestBody ListAuthenticatorOpsLogRequest listAuthenticatorOpsLogRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);

        ListAuthenticatorOpsLogsRequest listAuthenticatorOpsLogsRequest = new ListAuthenticatorOpsLogsRequest();
        listAuthenticatorOpsLogsRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        listAuthenticatorOpsLogsRequest.setUserId(userId);
        listAuthenticatorOpsLogsRequest.setPageSize(10L);

        List<AuthenticationOpsLog> authenticationOpsLogs = new ArrayList<>();
        ListAuthenticatorOpsLogsResponse listAuthenticatorOpsLogsResponse = iDaaSClientService.getClient().getAcsResponse(listAuthenticatorOpsLogsRequest);
        if (listAuthenticatorOpsLogsResponse.getAuthenticationLogContent() != null) {
            for (ListAuthenticatorOpsLogsResponse.AuthenticationLogContentItem item : listAuthenticatorOpsLogsResponse.getAuthenticationLogContent()) {
                AuthenticationOpsLog authenticationOpsLog = new AuthenticationOpsLog();
                authenticationOpsLog.setName(item.getAuthenticatorName());
                authenticationOpsLog.setAuthenticatorType(item.getAuthenticatorType());
                authenticationOpsLog.setOperator(item.getOperationAction());
                authenticationOpsLog.setOperationTime(item.getOperationTime());
                authenticationOpsLog.setSuccess(item.getOperationResult());
                authenticationOpsLog.setUserAgent(item.getUserAgent());
                authenticationOpsLog.setUserSourceIp(item.getUserSourceIp());
                authenticationOpsLogs.add(authenticationOpsLog);
            }
        }

        ListAuthenticatorOpsLogResponse listAuthenticatorOpsLogResponse = new ListAuthenticatorOpsLogResponse();
        listAuthenticatorOpsLogResponse.setAuthenticationOpsLogs(authenticationOpsLogs);

        return ResponseEntity.ok(listAuthenticatorOpsLogResponse);
    }
}
