package com.alibaba.cloud.security.sample.web;

import com.alibaba.cloud.security.sample.params.password.CheckPasswordRequest;
import com.alibaba.cloud.security.sample.params.password.CheckPasswordResponse;
import com.alibaba.cloud.security.sample.service.IDaaSClientService;
import com.aliyuncs.idaas_doraemon.model.v20210520.ListPwnedPasswordsRequest;
import com.aliyuncs.idaas_doraemon.model.v20210520.ListPwnedPasswordsResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpSession;
import java.util.HashMap;

@Api
@Controller
@RequestMapping("/password")
public class PasswordController {

    @Autowired
    IDaaSClientService iDaaSClientService;

    @ApiOperation(value = "CheckPassword", notes = "Check password leak")
    @ResponseBody
    @RequestMapping(value = "/check", method = RequestMethod.POST)
    public ResponseEntity<CheckPasswordResponse> check(@ApiIgnore HttpSession session, @RequestBody CheckPasswordRequest checkPasswordRequest) throws Exception {
        String passwordSha1Prefix = checkPasswordRequest.getPasswordSha1Prefix();
        if (!StringUtils.hasText(passwordSha1Prefix)) {
            CheckPasswordResponse checkPasswordResponse = new CheckPasswordResponse();
            checkPasswordResponse.setMessage("Password SHA1 prefix is empty.");
            return ResponseEntity.badRequest().body(checkPasswordResponse);
        }

        ListPwnedPasswordsRequest listPwnedPasswordsRequest = new ListPwnedPasswordsRequest();
        listPwnedPasswordsRequest.setPrefixHexPasswordSha1Hash(passwordSha1Prefix);

        ListPwnedPasswordsResponse listPwnedPasswordsResponse = iDaaSClientService.getClient().getAcsResponse(listPwnedPasswordsRequest);

        HashMap<String, Long> passwordSha1CountMap = new HashMap<>();
        if (listPwnedPasswordsResponse.getPwnedPasswordInfos() != null) {
            for (ListPwnedPasswordsResponse.Items pwnedPasswordInfo : listPwnedPasswordsResponse.getPwnedPasswordInfos()) {
                if (pwnedPasswordInfo.getHexPasswordSha1Hash() != null && pwnedPasswordInfo.getPwnedCount() != null) {
                    passwordSha1CountMap.put(pwnedPasswordInfo.getHexPasswordSha1Hash().toLowerCase(), pwnedPasswordInfo.getPwnedCount());
                }
            }
        }

        CheckPasswordResponse checkPasswordResponse = new CheckPasswordResponse();
        checkPasswordResponse.setPasswordSha1CountMap(passwordSha1CountMap);

        return ResponseEntity.ok(checkPasswordResponse);
    }
}
