package com.alibaba.cloud.security.sample.web;

import com.alibaba.cloud.security.sample.entity.UserDO;
import com.alibaba.cloud.security.sample.params.authentication.*;
import com.alibaba.cloud.security.sample.service.IDaaSClientService;
import com.alibaba.cloud.security.sample.service.MockUserService;
import com.alibaba.cloud.security.sample.util.LoginUtil;
import com.aliyuncs.idaas_doraemon.model.v20210520.*;
import com.google.gson.Gson;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpSession;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

@Api
@Controller
@RequestMapping("/totp-authentication")
public class TotpAuthenticationController {
    @Value("${app.idaas.applicationExternalId}")
    String alibabaCloudIdaasApplicationExternalId;

    @Autowired
    MockUserService mockUserService;
    @Autowired
    IDaaSClientService iDaaSClientService;

    @RequestMapping(value = "/initialize_create_credential", method = RequestMethod.POST)
    public ResponseEntity<InitializeCredentialResponse> initializeCreateCredential(@ApiIgnore HttpSession session, @RequestBody InitializeCreateCredentialRequest initializeCreateCredentialRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);
        UserDO user = null;
        if (StringUtils.hasText(userId)) {
            user = mockUserService.findUserByUserId(userId);
        }
        if (user == null) {
            return ResponseEntity.notFound().build();
        }

        Map<String, Object> otpCreateParams = new HashMap<>();
        otpCreateParams.put("algorithm", "HmacSHA1");
        otpCreateParams.put("keySeedRespMode", "GOOGLE_QR");
        otpCreateParams.put("qrCodeImgWidth", 200);
        otpCreateParams.put("qrCodeImgHeight", 200);
        otpCreateParams.put("period", 30);
        otpCreateParams.put("issuer", "webauthn-demo");

        CreateAuthenticatorRegistrationRequest createAuthenticatorRegistrationRequest = new CreateAuthenticatorRegistrationRequest();
        createAuthenticatorRegistrationRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        createAuthenticatorRegistrationRequest.setAuthenticatorType("TOTP");
        createAuthenticatorRegistrationRequest.setUserId(userId);
        createAuthenticatorRegistrationRequest.setUserName(user.getLoginId());
        createAuthenticatorRegistrationRequest.setUserDisplayName(user.getDisplayName());
        createAuthenticatorRegistrationRequest.setServerExtendParamsJson(
                Base64.getEncoder().encodeToString(
                        new Gson().toJson(otpCreateParams).getBytes(StandardCharsets.UTF_8)
                )
        );

        CreateAuthenticatorRegistrationResponse createAuthenticatorRegistrationResponse = iDaaSClientService.getClient().getAcsResponse(createAuthenticatorRegistrationRequest);

        session.setAttribute("TOTP-REGISTRATION-CHALLENGE", createAuthenticatorRegistrationResponse.getChallengeBase64());

        InitializeCredentialResponse initializeCredentialResponse = new InitializeCredentialResponse();
        initializeCredentialResponse.setChallengeBase64(createAuthenticatorRegistrationResponse.getChallengeBase64());
        initializeCredentialResponse.setOptions(createAuthenticatorRegistrationResponse.getOptions());
        return ResponseEntity.ok(initializeCredentialResponse);
    }

    @RequestMapping(value = "/create_credential", method = RequestMethod.POST)
    public ResponseEntity<CreateCredentialResponse> createCredential(@ApiIgnore HttpSession session, @RequestBody CreateCredentialRequest createCredentialRequest) throws Exception {
        String userId = LoginUtil.getLoginUserId(session);
        UserDO user = null;
        if (StringUtils.hasText(userId)) {
            user = mockUserService.findUserByUserId(userId);
        }
        if (user == null) {
            return ResponseEntity.notFound().build();
        }

        RegisterAuthenticatorRequest registerAuthenticatorRequest = new RegisterAuthenticatorRequest();
        registerAuthenticatorRequest.setApplicationExternalId(alibabaCloudIdaasApplicationExternalId);
        registerAuthenticatorRequest.setAuthenticatorType("TOTP");
        registerAuthenticatorRequest.setUserId(userId);
        registerAuthenticatorRequest.setAuthenticatorName("TOTP-DefaultName");
        registerAuthenticatorRequest.setRequireChallengeBase64((String) session.getAttribute("TOTP-REGISTRATION-CHALLENGE"));
        registerAuthenticatorRequest.setRegistrationContext(createCredentialRequest.getRegistrationContext());

        RegisterAuthenticatorResponse registerAuthenticatorResponse = iDaaSClientService.getClient().getAcsResponse(registerAuthenticatorRequest);

        String authenticatorUuid = registerAuthenticatorResponse.getAuthenticatorUuid();

        CreateCredentialResponse createCredentialResponse = new CreateCredentialResponse();
        createCredentialResponse.setAuthenticatorUuid(authenticatorUuid);
        return ResponseEntity.ok(createCredentialResponse);
    }
}
