async function doFetch(url, data) {
    var fetchParams = {
        method: 'POST',
        mode: 'same-origin',
        cache: 'no-cache',
        credentials: 'same-origin',
        headers: {
            'Content-Type': 'application/json'
        }
    };
    if (data !== null && typeof data !== 'undefined') {
        fetchParams.body = JSON.stringify(data);
    }
    return await fetch(url, fetchParams);
}

async function fetchUserInfo() {
    let userInfoResponse = await doFetch('/user/info');
    console.log(userInfoResponse);
    return userInfoResponse;
}

async function doLogout() {
    let logoutResponse = await doFetch('/user/logout');
    console.log(logoutResponse);
    return logoutResponse;
}

async function doLogoutAndReload() {
    let logoutResponse = await doFetch('/user/logout');
    console.log(logoutResponse);
    if (logoutResponse.status !== 200) {
        throw 'Logout failed: ' + logoutResponse.status;
    }
    location.reload();
}

async function doLogoutAndGoHome() {
    let logoutResponse = await doFetch('/user/logout');
    console.log(logoutResponse);
    if (logoutResponse.status !== 200) {
        throw 'Logout failed: ' + logoutResponse.status;
    }
    location.href = "/";
}

toastr.options = {
    "closeButton": true,
    "debug": false,
    "newestOnTop": false,
    "progressBar": false,
    "positionClass": "toast-top-right",
    "preventDuplicates": false,
    "onclick": null,
    "showDuration": "300",
    "hideDuration": "1000",
    "timeOut": "5000",
    "extendedTimeOut": "1000",
    "showEasing": "swing",
    "hideEasing": "linear",
    "showMethod": "fadeIn",
    "hideMethod": "fadeOut"
};

window.onunhandledrejection = event => {
    toastr["warning"](`${event.reason}`, 'System Error');
    console.warn(`UNHANDLED PROMISE REJECTION`, event);
};

window.onerror = function(message, source, lineNumber, colno, error) {
    toastr["warning"](`${message}, error: ${error || '-'}, source: ${source || 'N/A'} (${lineNumber || 'n/a'}:${colno || 'n/a'})`, 'System Error');
    console.warn(`UNHANDLED ERROR`, error);
};
