async function nextStep() {
    let loginId = $('#loginId').val();
    globalContext.loginId = loginId;
    let queryResponse = await doFetch('/user/query', {
        loginId: loginId
    });
    console.log(queryResponse);
    if (queryResponse.status === 404) {
        alert('User not exists.');
        return;
    }
    if (queryResponse.status !== 200) {
        alert('Server error: ' + queryResponse.status);
        return;
    }
    let signUpResponseJson = await queryResponse.json();
    globalContext.userId = signUpResponseJson.userId;
    if (signUpResponseJson.registeredWebAuthn && await IDaaSWebAuthnSDK.supportsPlatformWebAuthn()) {
        globalContext.step = 'webauthn';
        globalContext.showWebAuthn = true;
    }  else {
        globalContext.step = 'password';
        globalContext.showWebAuthn = false;
    }
    globalContext.showTotp = signUpResponseJson.openTotp || false;

    if (globalContext.showWebAuthn && !globalContext.idaaSWebAuthnSDK) {
        globalContext.idaaSWebAuthnSDK = new IDaaSWebAuthnSDK();
    }
    //return signUpResponseJson;
}

async function doPasswordLogin() {
    let password = $('#password').val();
    let loginResponse = await doFetch('/user/login', {
        loginId: globalContext.loginId,
        password: password,
        totpCode: document.getElementById('totp') && document.getElementById('totp').value || ''
    });
    if (loginResponse.status === 404) {
        alert('User not exists.');
        return;
    }
    if (loginResponse.status === 400) {
        let serverMessage = null;
        try {
            serverMessage = (await loginResponse.json()).message;
        } catch (e){ /*IGNORE*/ }
        alert(serverMessage || 'User not exists, password or totp error');
        return;
    }
    if (loginResponse.status !== 200) {
        alert(`Unknown error: ${loginResponse.status}`);
        return;
    }
    location.href = "/";
}

async function doWebAuthnLogin() {
    const authenticate = await globalContext.idaaSWebAuthnSDK.authenticate(globalContext.userId,async function (payload) {
        const initializeVerifyCredential = await doFetch('/authentication/initialize_verify_credential', payload)
        const initializeVerifyCredentialResponse = await initializeVerifyCredential.json()
        return {
            code:200,
            data:initializeVerifyCredentialResponse
        }
    },async function (payload) {
        const authenticationVerifyCredential = await doFetch('/authentication/verify_credential', payload)
        const authenticationVerifyCredentialResponse = await authenticationVerifyCredential.json()
        return {
            code:200,
            data:authenticationVerifyCredentialResponse
        }
    })
    if (authenticate) {
        location.href = "/";
    } else {
        alert('WebAuthn login failed');
    }
}

function switchLogin(step) {
    globalContext.step = step;
    return false;
}

